/*
 * Show.java This code is released to the public domain. Created by Mattias
 * Larsson, ml@techno.org Created on April 22, 2003, 5:31 PM
 */

package xmltv;

import java.io.ByteArrayOutputStream;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.zip.CRC32;
import java.util.Locale;

///////////////////////////////////////////////
import java.util.Vector;
import java.text.DateFormat;
import java.util.Calendar;
///////////////////////////////////////////////


/**
 * @author mta
 */
public class Show {


/////////////////////////////////////
	static String regionalChannel;
	//boolean states whether this is on a regional channel
	public boolean regional = false;
	private Channel rChannel;
	private sage.EPGDBPublic epg;
/////////////////////////////////////




	public String title;
	public String episodeName;
	public String desc;
	public int duration;
	public String category;
	public String subCategory;
	//	public String[] people;
	//    public byte[] roles;
	private LinkedList people = new LinkedList();
	private ByteArrayOutputStream roles = new ByteArrayOutputStream();
	public String rated;
	public String[] expandedRatings;
	public String year;
	public String parentalRating;
	public String[] bonus;
	public String extId;
	public String language;
	public long originalAirDate;
	public boolean rerun;

	public String startString;
	public String endString;

	public long start;
	public long length;

	//	public int chanId;
	public Channel channel;

	static public SimpleDateFormat formatterSeconds;
	static public SimpleDateFormat formatterNoSeconds;
	static Map categoryTranslations = new HashMap();
	private static final String[] ROLES = {null, "actor", "actor.lead",
			"actor.support", "actress", "actress.lead", "actress.support",
			"guest", "guest.star", "director", "producer", "writer",
			"choreographer", "sports.figure", "coach", "host",
			"producer.executive"};
	private static final List ROLES_LIST = Arrays.asList(ROLES);
	private static final String[] CREDITORS = {"director", "actor", "writer",
			"adapter", "producer", "presenter", "commentator", "guest"};
	static final List CREDITORS_LIST = Arrays.asList(CREDITORS);
	private static byte[] creditRoles = new byte[CREDITORS.length];

	static {
		formatterSeconds = new SimpleDateFormat("yyyyMMddHHmmss Z");
		formatterNoSeconds = new SimpleDateFormat("yyyyMMddHHmm Z");
	}

	/**
	 * Initialize this class with the xmltv.properties file.
	 *
	 * @param cfg the loaded xmltv.properties file
	 */
	static void init(Properties cfg) {
		initCategoryTranslations(cfg);
		initCreditRoles(cfg);
	}

	private static void initCategoryTranslations(Properties cfg) {
		String translateCategoryPrefix = "translate.category.";
		int translateCategoryPrefixLength = translateCategoryPrefix.length();

		categoryTranslations.clear();
		Iterator it = cfg.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
			String key = ((String) entry.getKey()).toLowerCase();
			if (key.startsWith(translateCategoryPrefix)) {
				categoryTranslations.put(key
						.substring(translateCategoryPrefixLength), entry
						.getValue());
			}
		}
		System.out.println("categoryTranslations = " + categoryTranslations);
	}

	private static void initCreditRoles(Properties cfg) {
		String creditsPrefix = "credits.";
		int creditsPrefixLength = creditsPrefix.length();
		creditRoles = new byte[CREDITORS.length];

		Iterator it = cfg.entrySet().iterator();
		while (it.hasNext()) {
			Map.Entry entry = (Map.Entry) it.next();
			String key = ((String) entry.getKey()).toLowerCase();
			if (key.startsWith(creditsPrefix)) {
				int creditor = CREDITORS_LIST.indexOf(key
						.substring(creditsPrefixLength));
				int role = ROLES_LIST.indexOf(((String) entry.getValue())
						.toLowerCase());
				if (creditor > -1 && role > 0) {
					creditRoles[creditor] = (byte) role;
				}
			}
		}
		System.out.print("creditRoles = {");
		for (int i = 0; i < CREDITORS.length; ++i) {
			if (creditRoles[i] > 0) {
				if (i > 0) {
					System.out.print(", ");
				}
				System.out.print(CREDITORS[i] + "=" + ROLES[creditRoles[i]]);
			}
		}
		System.out.println("}");
	}

	/** Creates a new instance of Show */
	public Show() {
		this.duration = 0;
		this.rerun = false; // assume shows are firstrun
		//		this.people = new String[0];
		//		this.roles = new byte[0];
		this.expandedRatings = new String[0];
		this.bonus = new String[0];
		this.originalAirDate = 0;
		this.length = 0;
		//		this.chanId = -1;
	}

//////////////////////////////////////////
	//Changed code
	public void setRegional(Channel channel, sage.EPGDBPublic e){
		epg = e;
		rChannel = channel;
		regional = true;
		//log("Regional channel found." + this.channel.toString());
		//programmeId = rProgramme.getId();
	}
///////////////////////////////////////////





	public void setChannel(Channel chan) {
		//		if (chan == null) {
		//			this.chanId = -1;
		//			return;
		//		}
		//        if (s.indexOf(" ") > -1) {
		//            String id = s.substring(0, s.indexOf(" ")).trim();
		//            try {
		//            this.chanId = Integer.parseInt(id);
		//            } catch (java.lang.NumberFormatException e) {
		//                System.out.println("fmt: "+e.toString()+" '"+id+"'");
		//                this.chanId = -1;
		//            }
		//        } else {
		//            this.chanId = -1;
		//        }
		//		this.chanId = chan.getChannelId();
		this.channel = chan;
	}

	public void setStart(String s) {
		if (s == null)
			return;
		this.startString = s;
		try {
			if (s.indexOf(" ") == 12) {
				this.start = Show.formatterNoSeconds.parse(s).getTime();
			} else {
				this.start = Show.formatterSeconds.parse(s).getTime();
			}
		} catch (java.text.ParseException e) {
			System.out.println("err: " + e.toString());
			e.printStackTrace(System.out);
		}
	}

	public void setEnd(String s) {
		if (s == null)
			return;
		this.endString = s;
		try {
			if (s.indexOf(" ") == 12) {
				this.length = Show.formatterNoSeconds.parse(s).getTime()
						- this.start;
			} else {
				this.length = Show.formatterSeconds.parse(s).getTime()
						- this.start;
			}
		} catch (java.text.ParseException e) {
			System.out.println("err: " + e.toString());
			e.printStackTrace(System.out);
		}
	}

	public void setCategory(String s) {
		if (this.category == null) {
			this.category = s;
		} else {
			this.subCategory = s;
		}
	}

	public void setTitle(String s) {
		if(regional) log("Regional channel found." + s);
		if((this.title) != null) this.title = s + "; " + this.title;
        else this.title = s;
	}





///////////////////////////////////////////////
	public void setDesc(String s) {
		//log("titel " + this.title);
        if(regional && (this.title).equals(regionalChannel)){
			log("Regional programs found for " + regionalChannel + ": ");

			Vector startTimes = new Vector();
			int start = -5;
			while(findNextStartTime(s.substring(start+5)) != -1){
				//XMLTVImportPlugin.printlnWithPrependedDate(pw, "start before : " + start);
				start += findNextStartTime(s.substring(start+5)) + 5;
				//XMLTVImportPlugin.printlnWithPrependedDate(pw, "start after  : " + start);
				startTimes.add(new Integer(start));
			}

			for(int i=0;i<startTimes.size()-1;i++){
				int st1 = ((Integer) startTimes.elementAt(i)).intValue();
				int st2= s.length();
				if((i+1) < startTimes.size()) st2 = ((Integer) startTimes.elementAt(i+1)).intValue();
				//XMLTVImportPlugin.printlnWithPrependedDate(pw, "rest : " + s.substring(st1));
				String sTime = s.substring(st1,st1 +5);
				String eTime = s.substring(st2,st2 +5);
				//XMLTVImportPlugin.printlnWithPrependedDate(pw, "rest : " + (st1+6) + " " + st2);
				String title;
				if((i+1) == startTimes.size()-1) st2 = s.length();
				if(findNextStartTime(s.substring(st1+6,st2))==0){
					title = s.substring(st1+12,st2);
				}else{
					title = s.substring(st1+6,st2);
				}


				//very dirty hack
				boolean dayExtra = false;
				if(sTime.equals("00.00")) dayExtra=true;

				//log("time: " + sTime +"-" + eTime);
				//log("title: " + title);
				Show rProgramme = new Show();


				rProgramme.setChannel(rChannel);

				//rChannel.addProgramme(rProgramme.getId(), rProgramme);
				rProgramme.setTitle(title);
				rProgramme.regional = true;



				Calendar scal = Calendar.getInstance(Locale.getDefault());
				scal.setTimeInMillis(this.start);
				if(dayExtra) scal.add(Calendar.DATE, 1);
				scal.set(Calendar.HOUR, (new Integer(sTime.substring(0,2))).intValue());
				scal.set(Calendar.MINUTE, (new Integer(sTime.substring(3,5))).intValue());
				long sTimeMillis = scal.getTimeInMillis();

				Calendar ecal = Calendar.getInstance(Locale.getDefault());
				ecal.setTimeInMillis(this.start);
				if(dayExtra) ecal.add(Calendar.DATE, 1);
				ecal.set(Calendar.HOUR, (new Integer(eTime.substring(0,2))).intValue());
				ecal.set(Calendar.MINUTE, (new Integer(eTime.substring(3,5))).intValue());
				long eTimeMillis = ecal.getTimeInMillis();
				//XMLTVImportPlugin.printlnWithPrependedDate(pw, "st: " + sTimeMillis);
				//XMLTVImportPlugin.printlnWithPrependedDate(pw, "et: " + eTimeMillis);

				if(eTimeMillis < sTimeMillis) eTimeMillis += 86400000;
				//XMLTVImportPlugin.printlnWithPrependedDate(pw, "int:" + (new Integer(eTime.substring(0,2))).intValue() + (new Integer(eTime.substring(3,5))).intValue());
				//XMLTVImportPlugin.printlnWithPrependedDate(pw, "st: " + (new Date(sTimeMillis)).toString());
				//XMLTVImportPlugin.printlnWithPrependedDate(pw, "et: " + (new Date(eTimeMillis)).toString());

				rProgramme.start = sTimeMillis;
				rProgramme.length = (int) (eTimeMillis-sTimeMillis);
				if(rProgramme!=null){
					rProgramme.EPGInsert(this.epg);
				    //XMLTVImportPlugin.printlnWithPrependedDate(pw, programme.getId());
            	}
			}



		}
        else if((this.desc) != null) this.desc = s + "; " + this.desc;
        else this.desc = s;
    }

	public int findNextStartTime(String s){
		//System.out.println("findNextStartTime("+ s +")");
		//find the first starttime
		int dotIdx = 0;
		boolean found=false;
		while(!found){
			//find the first dot
			dotIdx = s.indexOf(".", dotIdx);
			//if there is no more dot, we're done
			if(dotIdx == -1){
				found = true;
			}else
			//if the dot is surrounded by digits it is a starttime
			if(	(s.length() >= (dotIdx+2)) &&
				Character.isDigit(s.charAt(dotIdx-1))&&
				Character.isDigit(s.charAt(dotIdx-2))&&
				Character.isDigit(s.charAt(dotIdx+1))&&
				Character.isDigit(s.charAt(dotIdx+2)) )
			{
				found=true;
				dotIdx = dotIdx-2;
			}else dotIdx++;
		}
		//System.out.println("return: "+ dotIdx);
		return dotIdx;
	}



	public void setRerun(boolean r){
		rerun = r;
	}
/////////////////////////////////////////////////////////////////






	public void setEpisodeName(String s) {
		this.episodeName = s;
	}

	public void EPGInsert(sage.EPGDBPublic epg) {


		log(this.toString());
		// Translate category combinations
		translateCategories();

		// create extId
		CRC32 crc = new CRC32();
		if (this.title != null)
			crc.update(this.title.getBytes());
		if (this.episodeName != null) {
			crc.update(this.episodeName.getBytes());
		} else {
			if (this.startString != null) {
				crc.update(this.startString.getBytes());
			}
		}
		if (this.desc != null)
			crc.update(this.desc.getBytes());

		// apply logic here.. if we cant determine if this is a unique show (ep
		// name?)
		// then include showing time in the EP UID, so make it unique
		this.extId = "EP" + Long.toHexString(crc.getValue());

		if (!rerun) { // firstrun
			if (!epg
					.addShowPublic(null, this.title, this.episodeName,
							this.desc, this.duration, this.category,
							this.subCategory, this.getPeople(),
							this.getRoles(), this.rated, this.expandedRatings,
							this.year, this.parentalRating, this.bonus,
							this.extId, this.language, this.originalAirDate)) {
				System.out.println("add show failed 2");
			}
		} else { // rerun
			if (!epg
					.addShowPublic(this.title, null, this.episodeName,
							this.desc, this.duration, this.category,
							this.subCategory, this.getPeople(),
							this.getRoles(), this.rated, this.expandedRatings,
							this.year, this.parentalRating, this.bonus,
							this.extId, this.language, this.originalAirDate)) {
				System.out.println("add show failed");
			}
		}
		// code to add airing as well, later
		//        if (this.chanId > -1 && this.length > 0) {
		if (this.channel != null && this.length > 0) {
			//  System.out.println("Airing: "+this.chanId+" "+this.extId+" "+new
			// java.util.Date(this.start).toString());
			if (!epg.addAiringPublic(this.extId,
			//                    this.chanId,
					this.channel.getChannelId(), this.start, this.length)) {
				System.out.println("airing add failed");
			}
		}
	}

	/**
	 * Translate the categories according to the xmltv.properties configuration.
	 */
	private void translateCategories() {
		if (this.category != null) {
			String key = this.category.toLowerCase();
			if (this.subCategory != null) {
				key += "/" + this.subCategory.toLowerCase();
			}
			if (Show.categoryTranslations.containsKey(key)) {
				String value = (String) Show.categoryTranslations.get(key);
				int i = value.indexOf('/');
				if (i == -1) {
					this.category = (value.length() == 0 ? null : value);
					this.subCategory = null;
				} else {
					this.category = value.substring(0, i);
					this.subCategory = value.substring(i + 1);
				}
				return;
			}

			// We didn't have a translation for the combination, now try each
			// separately
			key = this.category.toLowerCase();
			if (Show.categoryTranslations.containsKey(key)) {
				String value = (String) Show.categoryTranslations.get(key);
				int i = value.indexOf('/');
				if (i == -1) {
					this.category = (value.length() == 0 ? null : value);
				} else {
					this.category = value.substring(0, i);
					this.subCategory = value.substring(i + 1);
					return;
				}
			}

			if (this.subCategory != null) {
				key = this.subCategory.toLowerCase();
				if (Show.categoryTranslations.containsKey(key)) {
					String value = (String) Show.categoryTranslations.get(key);
					int i = value.indexOf('/');
					if (i == -1) {
						this.subCategory = (value.length() == 0 ? null : value);
					} else {
						this.subCategory = value.substring(0, i);
						if (this.category.length() == 0) {
							this.category = this.subCategory;
							this.subCategory = value.substring(i + 1);
						}
					}
				}

				if (this.category.equals(this.subCategory)) {
					this.subCategory = null;
				}
			}
		}
	}

	/**
	 * @return
	 */
	public String[] getPeople() {
		return (String[]) people.toArray(new String[0]);
	}

	/**
	 * @return
	 */
	public byte[] getRoles() {
		return roles.toByteArray();
	}

	/**
	 * Adds a creditor to the show.
	 *
	 * @param creditor the creditor element
	 * @param name the name of the creditor
	 */
	public void setCreditor(int creditor, String name) {
		byte role = Show.creditRoles[creditor];
		if (role > 0 && name.length() > 0) {
			this.people.add(name);
			this.roles.write(role);
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append(this.channel
				+ " "
				+ new SimpleDateFormat("dd-MM-yyyy HH:mm").format(new Date(
						this.start))
				+ "," + this.length/60000 + "min"
				+ ": "
				+ this.title);
		if (this.episodeName != null) {
			sb.append(" - " + this.episodeName);
		}
		return sb.toString();
	}

	private void log(String s) {
			System.out.println(LDF.format(new Date()) + s);
	}
	private static final SimpleDateFormat LDF = new SimpleDateFormat(
				"dd-MM-yyyy HH:mm:ss,S ");

}