/*
 * XMLTVImportPlugin.java This code is released to the public domain. Created by
 * Mattias Larsson, ml@techno.org Created on April 21, 2003, 5:52 PM
 */

package xmltv;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import java.util.TreeMap;
//////////////////////////////////////////
import java.util.Vector;
import java.util.Map;
//////////////////////////////////////////
import org.apache.xerces.parsers.SAXParser;
import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.Locator;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.XMLReader;




/**
 * More information at: http://mta.techno.org/epg/
 *
 * @author mta
 */
public class XMLTVImportPlugin implements sage.EPGImportPlugin, ContentHandler,
		ErrorHandler {

	public String[][] prov = new String[][] {new String[] {"867507149",
			"XMLTV Lineup"}};

	private sage.EPGDBPublic epg;

	// XML parsing scratch
	private Channel channel;
	private StringBuffer text;
	private HashMap channels;
	private Show show;

///////////////////////////////////////////
	static String regionalChannel;
	static Vector badChannels = new Vector();
	private HashMap extraChannels = new HashMap();
//////////////////////////////////////////

	/** Creates a new instance of XMLTVImportPlugin */
	public XMLTVImportPlugin() {

//////////////////////////////////////////////
		//Show.regionalChannel = "TV WEST";
		//regionalChannel = "TV WEST";
////////////////////////////////////////////////

	}

//////////////////////////////////////////////
	private static void initBadChannels(Properties cfg) {
			String badChannelPrefix = "dismiss.channel.";
			int badChannelPrefixLength = badChannelPrefix.length();

			badChannels.clear();
			Iterator it = cfg.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry entry = (Map.Entry) it.next();
				String key = ((String) entry.getKey());
				//System.out.println("key = " + key);
				if (key.startsWith(badChannelPrefix)) {
					badChannels.add(key
							.substring(badChannelPrefixLength));
				}
			}
			System.out.println("badChannels = " + badChannels);
	}

	private void initExtraChannels(Properties cfg) {
			String extraChannelPrefix = "extra.channel.";
			int extraChannelPrefixLength = extraChannelPrefix.length();
			//extraChannels.clear();
			Iterator it = cfg.entrySet().iterator();
			while (it.hasNext()) {
				Map.Entry entry = (Map.Entry) it.next();
				String key = ((String) entry.getKey());
				//System.out.println("key = " + key);
				if (key.startsWith(extraChannelPrefix)) {
					Channel xC = new Channel();
					xC.setId(key.substring(extraChannelPrefixLength));
					xC.setDisplayName((String) entry.getValue());
					this.channels.put(xC.id, xC);
					xC.EPGInsert(this.epg);
					log("Extra channel added: " + xC.toString());
				}


			}

	}
///////////////////////////////////////////
	public String[][] getLocalMarkets() {
		return prov;
	}

	public String[][] getProviders(String str) {
		return prov;
	}

	PrintStream sysout = System.out;
	PrintStream syserr = System.err;
	PrintStream console;

	private void init() {
		File file = new File("xmltv.exceptions.log");
		if (file.exists()) {
			log("Removing " + file.getAbsolutePath());
			file.delete();
		}
		try {
			file = new File("xmltv.console.log");
			console = new PrintStream(new FileOutputStream(file));
			log("Rerouting console to " + file.getAbsolutePath());
			System.setOut(console);
			System.setErr(console);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			writeException(e);
		}
		try {

			file = new File("xmltv.properties");
			if (!file.exists()) {
				log(file.getAbsolutePath() + " not found, using defaults.");
				return;
			}

			Properties cfg = new Properties();
			try {
				FileInputStream stream = new FileInputStream(file);
				cfg.load(stream);
				stream.close();

				Show.init(cfg);
				////////////////////////////////////////////////////////
				initBadChannels(cfg);
				initExtraChannels(cfg);
				regionalChannel = cfg.getProperty("RegionalChannel");
				Show.regionalChannel = regionalChannel;
				////////////////////////////////////////////////////////
				String cmd = cfg.getProperty("run.before");
				if (null != cmd && cmd.trim().length() > 0) {
					Process process = Runtime.getRuntime().exec(cmd);
					sysout(process.getInputStream());
					System.err.println(toString(process.getErrorStream()));
				}
			} catch (Exception e) {
				e.printStackTrace();
				writeException(e);
				return;
			}
		} catch (Throwable t) {
			t.printStackTrace();
			writeException(t);
		}
	}

	private void exit() {
		if (console != null) {
			log("Restoring console.");
			System.setOut(sysout);
			System.setErr(syserr);
			console.close();
			log("Console restored.");
		}
	}

	private void sysout(InputStream stream) {
		byte[] buffer = new byte[1000];
		try {
			int bytesRead = stream.read(buffer);
			while (bytesRead > 0) {
				System.out.print(new String(buffer, 0, bytesRead));
				bytesRead = stream.read(buffer);
			}
		} catch (IOException e) {
			e.printStackTrace();
			writeException(e);
		}
	}

	private String toString(InputStream stream) {
		StringBuffer sb = new StringBuffer();
		byte[] buffer = new byte[1000];
		try {
			int bytesRead = stream.read(buffer);
			while (bytesRead > 0) {
				sb.append(new String(buffer, 0, bytesRead));
				bytesRead = stream.read(buffer);
			}
		} catch (IOException e) {
			e.printStackTrace();
			writeException(e);
		}
		return sb.toString();
	}

	/**
	 * @param e
	 */
	private void writeException(Throwable e) {
		try {
			FileOutputStream stream = new FileOutputStream(
					"xmltv.exceptions.log", true);
			try {
				e.printStackTrace(new PrintStream(stream));
			} finally {
				stream.close();
			}
		} catch (IOException e1) {
			e1.printStackTrace();
		}
		if (e instanceof OutOfMemoryError) {
			writeException(new Throwable("Location:"));
		}
	}

	private static final SimpleDateFormat LDF = new SimpleDateFormat(
			"dd-MM-yyyy HH:mm:ss,S ");

	private void log(String s) {
		System.out.println(LDF.format(new Date()) + s);
	}

	public boolean updateGuide(String str, sage.EPGDBPublic epg) {
		try {

			this.epg = epg;
			this.channels = new HashMap();

			init();

			try {
				XMLReader r = new SAXParser();
				r
						.setFeature(
								"http://apache.org/xml/features/nonvalidating/load-external-dtd",
								false);
				r.setContentHandler(this);
				r.setErrorHandler(this);
				try {
					r
							.parse(new InputSource(new FileInputStream(
									"epgdata.xml")));
				} catch (Exception e) {
					e.printStackTrace();
					writeException(e);
					return false;
				}

				TreeMap map = new TreeMap();
				int tunerchan = 2;
				for (Iterator i = this.channels.values().iterator(); i
						.hasNext();) {
					Channel c = (Channel) i.next();
					String ch;
					//log(c.toString());
					if (c.getTunerChannel() < 0) {
						// no real tuner channel available
						ch = "" + tunerchan;
						tunerchan++;
					} else { // real tuner available
						ch = "" + c.getTunerChannel();
					}
					map.put(new Integer(c.getChannelId()), new String[] {ch});
				}

				this.epg.setLineup(867507149L, map);
				// provider hardcoded since sagetv doesnt call get providers

			} catch (Exception e) {
				e.printStackTrace();
				writeException(e);
			} finally {
				exit();
			}
		} catch (Throwable t) {
			t.printStackTrace();
			writeException(t);
		}

		return true;
	}

	/* **** XML Parsing **** */
	public void characters(char[] ch, int start, int length)
			throws SAXException {
		if (this.text != null) {
			this.text.append(ch, start, length);
		}
	}

	public void endDocument() throws SAXException {
	}

	public void endElement(String namespaceURI, String localName, String qName)
			throws SAXException {
		String tag = qName.toLowerCase();
		boolean isShow = (this.show != null && this.text != null);
		//log((new Boolean(isShow)).toString());
		if (tag.equals("channel")) {
			if(!badChannels.contains(this.channel.id)){
				this.channels.put(channel.id, channel);
				this.channel.EPGInsert(this.epg);
				log("Channel added: " + this.channel.toString());
			}
			this.channel = null;
		} else if (tag.equals("programme") && this.show!= null) {
			// add show here...
			//log(this.show.toString());
			if(!badChannels.contains(this.show.channel.id)) this.show.EPGInsert(this.epg);
			this.show = null;
		} else if (tag.equals("sub-title") && isShow) {
			this.show.setEpisodeName(this.text.toString());
		} else if (tag.equals("desc") && isShow) {
			this.show.setDesc(this.text.toString());
		} else if (tag.equals("title") && isShow) {
			this.show.setTitle(this.text.toString());
		} else if (tag.equals("category") && isShow) {
			this.show.setCategory(this.text.toString());
		} else if (tag.equals("display-name") && this.channel != null) {
			this.channel.setDisplayName(this.text.toString());
		} else if (Show.CREDITORS_LIST.contains(tag) && isShow) {
			addPerson(Show.CREDITORS_LIST.indexOf(tag), this.text.toString());
			//System.out.println(this.text.toString());
		}
		this.text = null;
	}

	public void endPrefixMapping(String prefix) throws SAXException {
	}

	public void ignorableWhitespace(char[] ch, int start, int length)
			throws SAXException {
	}

	public void setDocumentLocator(Locator locator) {
	}

	public void skippedEntity(String name) throws SAXException {
	}

	public void startDocument() throws SAXException {
	}




//////////////////////////////////////////////////////////////
	public void addPerson(int role, String s){
		if (s.indexOf(",") != -1){
			String[] actors = s.split(",");
			for(int i=0; i < actors.length; i++){
				this.addPerson(role, actors[i].trim());
			}
			s = "";
		}
		if (s.indexOf(";") != -1){
			String[] actors = s.split(";");
			for(int i=0; i < actors.length; i++){
				this.addPerson(role, actors[i].trim());
			}
			s = "";
		}
		if (s.indexOf(" en ") != -1){
			String[] actors = s.split(" en ");
			for(int i=0; i < actors.length; i++){
				this.addPerson(role, actors[i].trim());
			}
			s = "";
		}
		if (s.endsWith(" e")){
			int ending = s.indexOf(" e");
			s = s.substring(0,ending);
		}
		if (s.endsWith("e.a")){
			int ending = s.indexOf("e.a");
			s = s.substring(0,ending);
		}
		if (s.endsWith("e.a.")){
			int ending = s.indexOf("e.a.");
			s = s.substring(0,ending);
		}
		if (s.length() > 3){
			log("New person: " + Show.CREDITORS_LIST.get(role) + ": " + s);
			this.show.setCreditor(role, s);
		}
	}
//////////////////////////////////////////////////////////////


	//    private Collection

	public void startElement(String namespaceURI, String localName,
			String qName, Attributes atts) throws SAXException {
		String tag = qName.toLowerCase();
		if (tag.equals("channel")) {
			this.channel = new Channel();
			this.channel.setId(atts.getValue("id"));
		} else if (tag.equals("programme")) {
			String channelAttr = atts.getValue("channel");
			if(!badChannels.contains(channelAttr)){
				//System.out.println("iep");
				this.show = new Show();
				this.show.setStart(atts.getValue("start"));
				this.show.setEnd(atts.getValue("stop"));
				Channel chan = null;
				if (channelAttr != null) {
					chan = (Channel) this.channels.get(channelAttr);
					if (chan != null) {
						this.show.setChannel(chan);
					} else { // never seen before channel, create it
						chan = new Channel();
						chan.setId(channelAttr);
						this.channels.put(channelAttr, chan);
						this.show.setChannel(chan);
						chan.EPGInsert(this.epg);
						log("new Channel added: " + chan.toString());
					}
				}
			}
			/////////////////////////////////////////
			else if(channelAttr.equals("tv-regionaal.todo")){
				this.show = new Show();
				this.show.setStart(atts.getValue("start"));
				this.show.setEnd(atts.getValue("stop"));
				Channel chan = new Channel();
				chan.setId(channelAttr);
				this.show.setChannel(chan);
				chan = null;
				chan = (Channel) this.channels.get(regionalChannel);
				if (chan == null) {
				 // never seen before channel, create it
					chan = new Channel();
					chan.setId(regionalChannel);
					this.channels.put(regionalChannel, chan);
					log("new Channel added: " + chan.toString());
					chan.EPGInsert(this.epg);
				}
				(this.show).setRegional(chan,this.epg);
			}else this.show = null;
			/////////////////////////////////////////


		} else if ((tag.equals("category")
				|| tag.equals("desc")
				|| tag.equals("title") || tag.equals("sub-title"))
				&& this.show != null) {
			this.text = new StringBuffer();
		} else if (tag.equals("display-name")) {
			this.text = new StringBuffer();
		} else if (tag.equals("previously-shown")) {
			this.show.setRerun(true);
		} else if (Show.CREDITORS_LIST.contains(tag)) {
			this.text = new StringBuffer();
		}
	}

	public void startPrefixMapping(String prefix, String uri)
			throws SAXException {
	}

	public void processingInstruction(String target, String data)
			throws SAXException {
	}

	/**
	 * Receive notification of a recoverable error.
	 * <p>
	 * This corresponds to the definition of "error" in section 1.2 of the W3C
	 * XML 1.0 Recommendation. For example, a validating parser would use this
	 * callback to report the violation of a validity constraint. The default
	 * behaviour is to take no action.
	 * </p>
	 * <p>
	 * The SAX parser must continue to provide normal parsing events after
	 * invoking this method: it should still be possible for the application to
	 * process the document through to the end. If the application cannot do so,
	 * then the parser should report a fatal error even if the XML 1.0
	 * recommendation does not require it to do so.
	 * </p>
	 * <p>
	 * Filters may use this method to report other, non-XML errors as well.
	 * </p>
	 *
	 * @param exception The error information encapsulated in a SAX parse
	 * exception.
	 * @exception org.xml.sax.SAXException Any SAX exception, possibly wrapping
	 * another exception.
	 * @see org.xml.sax.SAXParseException
	 */
	public void error(SAXParseException exception) throws SAXException {
		log(exception.toString());
		writeException(exception);
	}

	/**
	 * Receive notification of a non-recoverable error.
	 * <p>
	 * This corresponds to the definition of "fatal error" in section 1.2 of the
	 * W3C XML 1.0 Recommendation. For example, a parser would use this callback
	 * to report the violation of a well-formedness constraint.
	 * </p>
	 * <p>
	 * The application must assume that the document is unusable after the
	 * parser has invoked this method, and should continue (if at all) only for
	 * the sake of collecting addition error messages: in fact, SAX parsers are
	 * free to stop reporting any other events once this method has been
	 * invoked.
	 * </p>
	 *
	 * @param exception The error information encapsulated in a SAX parse
	 * exception.
	 * @exception org.xml.sax.SAXException Any SAX exception, possibly wrapping
	 * another exception.
	 * @see org.xml.sax.SAXParseException
	 */
	public void fatalError(SAXParseException exception) throws SAXException {
		log(exception.toString());
		writeException(exception);
	}

	/**
	 * Receive notification of a warning.
	 * <p>
	 * SAX parsers will use this method to report conditions that are not errors
	 * or fatal errors as defined by the XML 1.0 recommendation. The default
	 * behaviour is to take no action.
	 * </p>
	 * <p>
	 * The SAX parser must continue to provide normal parsing events after
	 * invoking this method: it should still be possible for the application to
	 * process the document through to the end.
	 * </p>
	 * <p>
	 * Filters may use this method to report other, non-XML warnings as well.
	 * </p>
	 *
	 * @param exception The warning information encapsulated in a SAX parse
	 * exception.
	 * @exception org.xml.sax.SAXException Any SAX exception, possibly wrapping
	 * another exception.
	 * @see org.xml.sax.SAXParseException
	 */
	public void warning(SAXParseException exception) throws SAXException {
		log(exception.toString());
		writeException(exception);
	}
}